import traceback

import discord
from discord import Embed
from django.utils import timezone

from apps.alert_wing.managers.base_manager import BaseManager


class DiscordEmbedManager(BaseManager):
    """
    A class for managing Discord embed messages for exception handling.

    Args:
        exc (Exception): The exception instance.
        exc_id (str | int | None, optional): ID associated with the exception. Defaults to None.
        title (str, optional): Title of the embed. Defaults to an empty string.
        description (str, optional): Description of the embed. Defaults to an empty string.
        embed_color (discord.Color | None, optional): Color of the embed. Defaults to None.
        embed (discord.Embed | None, optional): Predefined discord.Embed object. Defaults to None.

    Attributes:
        exc (Exception): The exception instance.
        exc_id (str | int | None): ID associated with the exception.
        title (str): Title of the embed.
        description (str): Description of the embed.
        embed_color (discord.Color): Color of the embed.
        embed (discord.Embed): Discord embed object.

    Methods:
        setup_embed(): Sets up the discord.Embed object.
        _frame_summary_handler(frame_summary): Handles the formatting of a frame summary.
        format_exception(): Formats the exception and returns formatted text and extra detail.
        _embed_footer_data(): Generates data for the embed footer.
        set_data_for_delivery(): Prepares and returns the embed data as a dictionary.
    """

    def __init__(
        self,
        exc: Exception,
        exc_id: str | int | None = None,
        title: str = "",
        description: str = "",
        embed_color=None,
        embed: discord.Embed | None = None,
    ):
        if not exc:
            raise ValueError(
                "Exception instance must be provide (exc can't be None or falsy value)"
            )

        self.exc = exc
        self.exc_id = exc_id

        self.title = title
        self.description = description

        if embed_color:
            self.embed_color = embed_color
        else:
            self.embed_color = discord.Color.red()

        if embed:
            self.embed = embed
        else:
            self.embed = self.setup_embed()

    def setup_embed(self) -> discord.Embed:
        return Embed(
            title=self.title, description=self.description, colour=self.embed_color
        )

    def _frame_summary_handler(self, frame_summary):
        """
        Handle the formatting of a frame summary.

        Args:
            frame_summary (traceback.FrameSummary): Frame summary object.

        Returns:
            str: Formatted frame summary.
        """
        stack_summary_handler = traceback.StackSummary()
        return stack_summary_handler.format_frame_summary(frame_summary=frame_summary)

    def format_exception(self) -> tuple[str, str]:
        """
        Format the exception and return formatted text and extra detail.

        Returns:
            tuple[str, str]: Formatted exception and extra detail.
        """
        request = self.exc.__traceback__.tb_frame.f_locals.get("request")
        frame_summary_list = traceback.extract_tb(self.exc.__traceback__)[-2:]

        formatted_exc = (
            f"{str(self.exc)}\n"
            f"{self._frame_summary_handler(frame_summary_list[0])}\n"
            f"{self._frame_summary_handler(frame_summary_list[1])}\n"
        )
        extra_detail = (
            f"Request API  -> {request.build_absolute_uri() if request is not None else None}\n"
            f"Request Method -> {request.method}"
        )
        return formatted_exc, extra_detail

    def _embed_footer_data(self):
        data = f"DateTime: {timezone.now()}"
        if self.exc_id:
            data = data + f" Error ID : {self.exc_id}"
        return data

    def set_data_for_delivery(self) -> dict:
        """
        Prepare and return the embed data as a dictionary.

        Returns:
            dict: Embed data as a dictionary.
        """
        formatted_exception, extra_detail = self.format_exception()
        self.embed.add_field(
            name=str(self.exc.__class__.__name__), value=formatted_exception
        )
        self.embed.add_field(name="Extra Detail", value=extra_detail, inline=False)
        self.embed.set_footer(text=self._embed_footer_data())
        return self.embed.to_dict()



import json
from typing import Any

import requests
from django.conf import settings

from apps.alert_wing.senders.base import BaseSender


class Discord(BaseSender):
    """
    A class for sending messages or files to a Discord webhook.

    Attributes:
        DATA (dict): A class-level dictionary to store the message data.

    Args:
        content (str): The text content of the message. If `None`, either `file` or `embeds` must be provided.
        username (str): The username to display for the message sender.
        avatar_url (str): The URL of the avatar image to display for the message sender.
        tts (bool): Specifies if the message should be sent as a text-to-speech message.
        file (file-like object): A file-like object to upload and send along with the message.
        embeds (list): A list of Discord embed objects to include with the message.
        allowed_mentions (dict): A dictionary specifying the allowed mentions in the message.
        webhook (str): The URL of the Discord webhook to send the message to.

    Raises:
        ValueError: If neither `content`, `file`, nor `embeds` are provided.
        ValueError: If `webhook` is not provided.

    Methods:
        post():
            Sends the constructed message or file to the specified Discord webhook.

    Usage example:
        discord = Discord(
            content="Hello, everyone!",
            username="MyBot",
            avatar_url="https://example.com/avatar.png",
            tts=False,
            file=open("example.txt", "rb"),
            embeds=[{
                "title": "Example Embed",
                "description": "This is an example embed."
            }],
            allowed_mentions={"users": [1234567890]},
            webhook="https://discord.com/api/webhooks/1234567890/abcdefgh"
        )
        response = discord.post()
        print(response.status_code)
    """

    DATA = {}

    def __init__(
        self,
        content: str = None,
        username: str = None,
        avatar_url: str = None,
        tts: bool = False,
        file: Any = None,
        embeds: list = None,
        allowed_mentions=None,
    ):
        """
        Initializes a new Discord message or file.

        Args:
            content (str): The text content of the message. If `None`, either `file` or `embeds` must be provided.
            username (str): The username to display for the message sender.
            avatar_url (str): The URL of the avatar image to display for the message sender.
            tts (bool): Specifies if the message should be sent as a text-to-speech message.
            file (file-like object): A file-like object to upload and send along with the message.
            embeds (list): A list of Discord embed objects to include with the message.
            allowed_mentions (dict): A dictionary specifying the allowed mentions in the message.
            webhook (str): The URL of the Discord webhook to send the message to.

        Raises:
            ValueError: If neither `content`, `file`, nor `embeds` are provided.
            ValueError: If `webhook` is not provided.
        """
        if content is None and file is None and embeds is None:
            raise ValueError("Required one of content, file, embeds")

        if content is not None:
            self.DATA["content"] = content

        if username is not None:
            self.DATA["username"] = username

        if avatar_url is not None:
            self.DATA["avatar_url"] = avatar_url

        if embeds is not None:
            self.DATA["embeds"] = embeds

        if allowed_mentions is not None:
            self.DATA["allowed_mentions"] = allowed_mentions

        self.file = file

        try:
            self.webhook = settings.DISCORD_WEBHOOK_URL
        except AttributeError:
            raise ValueError("DISCORD_WEBHOOK_URL variable must set in django settings")

        self.DATA["tts"] = tts

    def send(self):
        """
        Sends the constructed message or file to the specified Discord webhook.

        Returns:
            The response
        """
        if self.file is not None:
            return requests.post(
                self.webhook, {"payload_json": json.dumps(self.DATA)}, files=self.file
            )

        return requests.post(
            self.webhook,
            json.dumps(self.DATA),
            headers={"Content-Type": "application/json"},
        )



from abc import ABC, abstractmethod


class BaseManager(ABC):
    @abstractmethod
    def format_exception(self) -> tuple[str, str]:
        raise NotImplementedError

    @abstractmethod
    def set_data_for_delivery(self):
        ...
